------------------------------------------------------------------------------------------
--
--  File     :  /mods/4DC_WOF_V0.8/lua/defaultunits.lua
--
--  Author(s):  Resin_Smoker, Domino
--
--  Summary  :  Scripts added for Weight of Fire (W.O.F.)
--
--  Note     :  Based upon earlier work done for SC by "Jotto"
--    
--  Copyright  2011 4DC_V0.8  All rights reserved.
--
------------------------------------------------------------------------------------------ 

do
local Custom_WOF_EffectTemplate = import('/mods/4DC_WOF_V0.8/lua/WOF_EffectTemplates.lua')

------------------------------------------------------------------------------------------
-- Air Units
------------------------------------------------------------------------------------------

local oldAirUnit = AirUnit
AirUnit = Class(oldAirUnit) {

    ShowUnitDestructionDebris = true,
    DestroyNoFallRandomChance = 1,
    
    OnImpact = function(self, with, other)
        -- Ensure OnImpact only runs once during death
        if self:IsDead() and not self.OnImpactBounce then
            self.OnImpactBounce = 1   
            -- Damage the area we have impacted
            local bp = self:GetBlueprint()
            local i = 1
            local numWeapons = table.getn(bp.Weapon)
            for i, numWeapons in bp.Weapon do
                if(bp.Weapon[i].Label == 'DeathImpact') then
                    DamageArea(self, self:GetPosition(), bp.Weapon[i].DamageRadius, bp.Weapon[i].Damage, bp.Weapon[i].DamageType, bp.Weapon[i].DamageFriendly)
                    break
                end
            end
            if with == 'Water' then 
                for k, v in Custom_WOF_EffectTemplate.WaterImpactRing do
                    CreateEmitterAtEntity( self, self:GetArmy(), v ):ScaleEmitter( explosion.GetAverageBoundingXYZRadius(self) )                    
                end            
                local pos = self:GetPosition()
                local surface = GetSurfaceHeight(pos[1], pos[3]) + GetTerrainTypeOffset(pos[1], pos[3])
                self:PlayUnitSound('AirUnitWaterImpact')                
                if table.find(self:GetBlueprint().Categories,'EXPERIMENTAL') then
                    -- HUGE water impact FX at each bone 
                    for i = 0, self:GetBoneCount() do
                        if self:IsValidBone(i) then
                            local bonePos = self:GetPosition(i) 
                            local offset
                            if bonePos[2] < surface then
                                offset = surface - bonePos[2]
                            else
                                offset = 0
                            end 
                            for k, v in Custom_WOF_EffectTemplate.ExperimentalWaterImpact do                        
                                CreateEmitterAtBone( self, i, self:GetArmy(), v):ScaleEmitter( explosion.GetAverageBoundingXYZRadius(self) ):OffsetEmitter(0,offset,0) 
                            end
                        end
                    end
                else
                    for k, v in EffectTemplate.DefaultProjectileWaterImpact do
                        CreateEmitterAtEntity( self, self:GetArmy(), v ):ScaleEmitter( explosion.GetAverageBoundingXYZRadius(self) * 2 )                    
                    end
                end                  
            else
                self:PlayUnitSound('Destroyed')
            end
            -- This is a bit of safety to keep us from calling the death thread twice in case we bounce twice quickly
            if not self.DeathBounce then
                self:ForkThread(self.DeathThread, self.OverKillRatio )
                self.DeathBounce = 1
            end
        end
    end,   

} 

end     