------------------------------------------------------------------------------------------
--
--	File		\mods\4DC_WOF_V0.8\hook\lua\selfdestruct.lua
--
--	Author(s)	Resin_Smoker
--
--	Co-Auth		Brute51, Domino
--
--	Updated		10/12/2013
--
--	Summary		Scripts added for Weight of Fire (W.O.F.) to pass signifigant data from
--				the projectile to the impacted unit
--
--	Copyright  2013 4DC_WOF_V0.8 WOF, All rights reserved.
--
------------------------------------------------------------------------------------------

do--(start of non-destructive hook)

-- Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
local masterDebug = false

-- Custom files imported
local WOF_UNIT_UTILS = import('/mods/4DC_WOF_V0.8/lua/WOF_UNIT_UTILS.lua')

-- Misc local files
local utilities = import('/lua/utilities.lua')

local oldProjectile = Projectile

Projectile = Class(oldProjectile) {

	PassDamageData = function( self, damageData ) -- Verified good 9/14/2013
		local myDebug = false
		-- Ensure that data from weapon.lua is being passed into projectile DamageData
		oldProjectile.PassDamageData(self, damageData)
		if myDebug or masterDebug then
			LOG('*** Projectile.lua, PassDamageData ***')
		end
		if self.DamageData then
			if myDebug or masterDebug then
				LOG('	DamageData being passed:')
			end
			-- Ensure that projectiles used as props dont trigger the passing of data
			if self.DamageData.DamageAmount > 1  or self.DamageData.WEAP.NukeWeapon then
				-- Pass data between weapon and projectile tables
				self.DamageData = damageData
				if self.DamageData.WEAP.NukeWeapon then
					LOG('	Damage amount based on Nuke detonation effects')
				elseif myDebug or masterDebug then
					LOG('	Damage amount > 1')
				end
			else	
				if myDebug or masterDebug then
					LOG('	Damage amount < 1 or projectile is being used for effects and thus not a valid trigger for WOF')
				end
			end
		
		elseif myDebug or masterDebug then
			LOG('	 No DamageData')
		end
	end,

	OnImpact = function(self, targetType, targetEntity) -- Verified good 9/14/2013
		local myDebug = false
		-- Pass projectile DamageData to target or targets impacted
		if myDebug or masterDebug then
			LOG('*** Projectile.lua, OnImpact ***')
			if not targetEntity then
				LOG('	TargetEntity not given or projectile impacted ground')
			end
		end
		if self and not self:BeenDestroyed() and self.DamageData then
			local bp = self:GetBlueprint()
			--Add PROJ data to DamageData table
			self.DamageData.PROJ = {}
			self.DamageData.PROJ.DMG_AMOUNT = self.DamageData.DamageAmount
			self.DamageData.PROJ.DMG_RADIUS = self.DamageData.DamageRadius
			self.DamageData.PROJ.DMG_TYPE = self.DamageData.DamageType
			self.DamageData.PROJ.IMPACT_LOC = self:GetPosition()
			self.DamageData.PROJ = bp or 'UNKNOWN'
			self.DamageData.PROJ.TICK = GetGameTick()			
			-- Ensure that projectiles used as props dont trigger the passing as data
			if self.DamageData.DamageAmount > 1 then
				if not self.DamageData.WEAP.DoTTime or self.DamageData.WEAP.DoTTime <= 0 then
					local radius = self.DamageData.DamageRadius
					if radius and radius > 0 then
						if myDebug or masterDebug then
							LOG('	Radius > 0, passing damage data to multible targets')
							LOG('	', repr(self.DamageData) )
						end
						local units = utilities.GetEnemyUnitsInSphere(self, self:GetPosition(), radius)
						if table.getsize(units) > 0 then
							for k, v in units do
								if not v:BeenDestroyed() and IsUnit(v) and not v.WOF_STUFF.UNIT.ACTIVE then
									WOF_UNIT_UTILS.WOF_PASS_PROJ_DATA_TO_UNIT( self, self.DamageData, v)
								end
							end
						end
					elseif targetEntity and not targetEntity:BeenDestroyed() and IsUnit(targetEntity) and not targetEntity.WOF_STUFF.UNIT.ACTIVE then
						if myDebug or masterDebug then
							LOG('	No radius defined, passing damage data to single target')
							LOG('	', repr(self.DamageData) )
						end
						WOF_UNIT_UTILS.WOF_PASS_PROJ_DATA_TO_UNIT( self, self.DamageData, targetEntity)
					end
				end
			elseif myDebug or masterDebug then
				LOG('	DamageAmount < 1 or projectile is being used for effects and thus not a valid trigger for WOF')
			end
		elseif myDebug or masterDebug then
			LOG('	Either no Self or Been Destroyed or no DamageData or Projectile is a Prop')
		end
		oldProjectile.OnImpact(self, targetType, targetEntity)
	end,

}
end--(end of non-destructive hook)