------------------------------------------------------------------------------------------
--
--	File	 :	\mods\4DC_WOF_V0.8\hook\lua\sim\Unit.lua
--
--	Author(s):	Resin_Smoker
--
--	Co-Auth:	Brute51, Domino
--
--	Updated:	10/20/2013
--
--	Summary  :	Scripts added for Weight of Fire (W.O.F.) to setup WOF conditions.
--
--	Copyright  2013 4DC_WOF_V0.8 WOF, All rights reserved.
--
------------------------------------------------------------------------------------------

do--(start of non-destructive hook)

-- Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
local masterDebug = true

local Custom_WOF_EffectTemplate = import('/mods/4DC_WOF_V0.8/lua/WOF_EffectTemplates.lua')
local WOF_UNIT_UTILS = import('/mods/4DC_WOF_V0.8/lua/WOF_UNIT_UTILS.lua')

local oldUnit = Unit
Unit = Class(oldUnit) {

	OnCreate = function( self )-- Verified good 10/16/2013
		local myDebug = false
		--Run oldUnit first			
		oldUnit.OnCreate( self )

		--Bypass units that are to be ignored
		local bp = self:GetBlueprint()		
		if bp and not bp.WOF.IGNORE then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('*** UNIT.LUA, OnCreate ***')		
			end								
			WOF_UNIT_UTILS.WOF_ON_CREATE( self, bp )
		end
	end,
	
    OnDestroy = function(self)
		local myDebug = false 
		if not self.WOF_STUFF.UNIT.WOF.IGNORE then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('*** UNIT.LUA, OnDestroy ***')		
			end								
			WOF_UNIT_UTILS.WOF_ON_DESTROY( self )
		else
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('*** UNIT.LUA, oldUnit OnDestroy ***')		
			end			
			--Run oldUnit last	      
	        oldUnit.OnDestroy(self)		
		end
    end,
    
    EnableUnitIntel = function(self, intel)
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, EnableUnitIntel ***')						
		end
		
		if not self:BeenDestroyed() then
			--Run oldUnit		 	
			oldUnit.EnableUnitIntel(self, intel)		
		end
    end,     	
		
	DoTakeDamage = function(self, instigator, amount, vector, damageType)
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, DoTakeDamage ***')						
		end
		WOF_UNIT_UTILS.DO_TAKE_DAMAGE(self, instigator, amount, vector, damageType) 
		--Run oldUnit		 	
		oldUnit.DoTakeDamage( self, instigator, amount, vector, damageType )
	end,		
	
    OnDamage = function( self, instigator, amount, vector, damageType )-- Verified good 10/16/2013
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, OnDamage ***')						
		end 
	
		if self:BeenDestroyed() or self:IsDead() or self.WOF_STUFF.UNIT.ACTIVE then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Unit is destroyed, dead, or WOF active, aborting function')							
			end
			return				
		else
			WOF_UNIT_UTILS.WOF_ON_DAMAGE( self, instigator, amount, vector, damageType )
			--Run oldUnit	
			oldUnit.OnDamage( self, instigator, amount, vector, damageType )		
		end
    end,   
        
	OnKilled = function(self, instigator, type, overkillRatio)
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, OnKilled ***')						
		end			
		if WOF_UNIT_UTILS.ON_KILLED( self, instigator, type, overkillRatio ) then
			--Run oldUnit
			oldUnit.OnKilled( self, instigator, type, overkillRatio )
		end
	end,
	
	DoDeathWeapon = function( self, overkillRatio )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, DoDeathWeapon ***')						
		end
		if WOF_UNIT_UTILS.DO_DEATH_WEAPON( self, overkillRatio ) then
			--Run oldUnit
			oldUnit.DoDeathWeapon( self, overkillRatio )		
		end
	end,	
		
	PlayAnimationThread = function( self, anim, rate )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, PlayAnimationThread ***')						
		end	
		if WOF_UNIT_UTILS.PLAY_ANIMATION_THREAD( self, anim, rate ) then
			--Run oldUnit		
			oldUnit.PlayAnimationThread( self, anim, rate )		
		end
	end,
	
	DeathThread = function( self, overkillRatio, instigator )		  
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, DeathThread ***')						
		end	
		if WOF_UNIT_UTILS.DEATH_THREAD( self, overkillRatio, instigator ) then
			--Run oldUnit		
			oldUnit.DeathThread( self, overkillRatio, instigator )		
		end
	end,
	    
    OnLayerChange = function( self, new, old )-- Verified good 10/16/2013
        local myDebug = false
        if self.WOF_STUFF then
	    	WOF_UNIT_UTILS.ON_LAYER_CHANGE( self, new, old  )        
	        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
	            LOG('*** UNIT.LUA, OnLayerChange ***')
	            LOG('New layer: ', self.WOF_STUFF.UNIT.LAYER )
	        end        
	    end
		--Run oldUnit last	    	 
        oldUnit.OnLayerChange( self, new, old )                            
    end,
    	
	OnTransportAttach = function( self, attachBone, unit )-- Verified good 10/16/2013
		local myDebug = false
		 --Run oldUnit first
		oldUnit.OnTransportAttach( self, attachBone, unit )
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, OnTransportAttach ***')
			LOG( WOF_UNIT_UTILS.WOF_ID_MY_UNIT( unit ) )				
		end		   	   
		self.WOF_STUFF.UNIT.IN_TRANSPORT = true	
	end,
	
	OnTransportDetach = function( self, attachBone, unit )-- Verified good 10/16/2013
		local myDebug = false
		--Run oldUnit first	
		oldUnit.OnTransportDetach( self, attachBone, unit )
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, OnTransportDetach ***')
			LOG( WOF_UNIT_UTILS.WOF_ID_MY_UNIT( unit ) )										
		end		   	   
		self.WOF_STUFF.UNIT.IN_TRANSPORT = false	   
	end,
	
	
    OnMotionHorzEventChange = function( self, new, old )
        oldUnit.OnMotionHorzEventChange( self, new, old )       
		local myDebug = false    
		if not self.WOF_STUFF.UNIT.WOF.IGNORE then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('*** UNIT.LUA, OnMotionHorzEventChange ***')		
			end								
			WOF_UNIT_UTILS.WOF_ON_MOTION_EVENT_CHANGE( self, new, old )
		end
    end,
    
	CreateWreckageProp = function( self, overkillRatio )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** UNIT.LUA, CreateWreckageProp ***')		
		end								
		WOF_UNIT_UTILS.WOF_CREATE_WRECKAGE_PROP( self, overkillRatio )		 
	end,    		

--
--	   DoTakeDamage = function(self, instigator, amount, vector, damageType)
--		local myDebug = false
--			if myDebug or masterDebug then
--			   LOG('*** Unit.lua, DoTakeDamage ***')
--			   WOF_UNIT_UTILS.WOF_ID_MY_UNIT( self )
--		   end
--		   -- Ensure that DoTakeDamage can no longer run after units health reaches "ZERO"
--		   if not self.WOF_DoTakeDamage_Flag then
--			   local preAdjHealth = self:GetHealth()
--			   self:AdjustHealth(instigator, -amount)
--			   local health = self:GetHealth()
--			   if( health <= 0 ) then
--				   -- Set flag to prevent event from running more then once
--				   self.WOF_DoTakeDamage_Flag = true
--				   if( damageType == 'Reclaimed' ) then
--					   self:Destroy()
--				   else
--					   -- If our unit is a transport, detach all cargo
--					   if self and EntityCategoryContains( categories.TRANSPORTATION, self ) then
--						   local cargo = self:GetCargo()
--						   for k, v in cargo do
--							   v:DetachFrom(false)
--						   end
--					   end
--					   -- If our unit is in transport, release it for WOF event
--					   if self.IsAttached_Flag and not self.WOF_Active_Flag then
--						   self:DetachFrom(true)
--					   end
--					   -- Calculate the excess damage amount
--					   local overkillRatio = 0.0
--					   local excess = preAdjHealth - amount
--					   local maxHealth = self:GetMaxHealth()
--					   if(excess < 0 and maxHealth > 0) then
--						   overkillRatio = -excess / maxHealth
--					   end
--					   -- Ensure that an instigator is always passed
--					   if not instigator then
--						   instigator = self
--					   end
--					   -- Pass data to WOF globals
--					   if vector and overkillRatio and self.WOF_Allowed_Flag then
--						   self.INSTIGATOR = instigator
--						   self.WOF_Vector = vector
--						   self.WOF_OverKill = overkillRatio
--					   else
--						   -- If data not avalible prevent WOF from triggering but allow a normal death event
--						   self.WOF_Allowed_Flag = false
--						   if not vector then
--							   WARN('WOF DoTakeDamage, "vector" information not avalible for unit ', self:GetUnitId())
--						   elseif not overkillRatio then
--							   WARN('WOF DoTakeDamage, "overkill" ratio not avalible for unit ', self:GetUnitId())
--						   end
--					   end
--					   self:Kill(instigator, damageType, overkillRatio)
--				   end
--			   end
--			   if EntityCategoryContains(categories.COMMAND, self) then
--				   local aiBrain = self:GetAIBrain()
--				   if aiBrain then
--					   aiBrain:OnPlayCommanderUnderAttackVO()
--				   end
--			   end
--		   end
--	   end,
--
--
--	   DoDeathWeapon = function(self, overkillRatio)
--		   -- Bypass death weapon if still being built or WOF triggered
--		   if self:IsBeingBuilt() or self.WOF_Active_Flag then
--			   --WARN('DoDeathWeapon, bypassed')
--			   return
--		   else
--			   -- Ensure that DoDeathWeapon can run only once per unit
--			   if not self.WOF_DoDeathWeapon_Flag then
--				   self.WOF_DoDeathWeapon_Flag = true
--				   if not table.find(self:GetBlueprint().Categories,'AIR') then
--					   local bp = self:GetBlueprint()
--					   local DeathWepFire = false
--					   for k, v in bp.Weapon do
--						   if(v.Label == 'DeathWeapon') then
--							   if v.FireOnDeath == true then
--								   self:SetWeaponEnabledByLabel('DeathWeapon', true)
--								   self:GetWeaponByLabel('DeathWeapon'):Fire()
--								   -- WARN('DoDeathWeapon, active')
--							   else
--								   self:ForkThread(self.DeathWeaponDamageThread, v.DamageRadius, v.Damage, v.DamageType, v.DamageFriendly)
--							   end
--							   WepFire = true
--							   break
--						   end
--					   end
--					   -- Area DMG Restrictions
--					   local DMG_Area_Restrict = {
--						  'AIR',
--						  'BLACKHOLE',
--						  'COMMAND',
--						  'EXPERIMENTAL',
--						  'HOLOGRAM',
--						  'POD',
--						  'NAVAL',
--						  'SUBCOMMANDER',
--						  'UNTARGETABLE',
--					   }
--					   local unitFilter = categories.MOBILE
--					   for k, v in DMG_Area_Restrict do
--						   unitFilter = unitFilter - ParseEntityCategory(v)
--					   end
--					   -- Allow Area DMG if conditions met
--					   if not DeathWepFire and EntityCategoryContains(unitFilter, self) then
--						   local bpMass = self:GetBlueprint().Economy.BuildCostMass
--						   self.WOF_DoDeathWeapon_Flag = true
--						   -- WARN('DoDeathWeapon, DamageArea', bpMass)
--						   -- Do the units mass in damage to the surrounding area
--						   self:ForkThread(self.DeathWeaponDamageThread, math.ceil(explosion.GetUnitVolume(self)), math.ceil(bpMass), 'Normal', true)
--					   end
--				   end
--			   else
--				   WARN('WOF DoDeathWeapon, attempted to run mutlible times for unit ', self:GetUnitId())
--			   end
--		   end
--	   end,
--
--	   OnLayerChange = function(self, new, old)
--		   -- Destroy the unit under WOF once it comes back down
--		   if self:IsDead() and not self:BeenDestroyed() and new != old and not self.IsAttached_Flag then
--			   -- Ensure that overkill is passed
--			   local overkillRatio = self:GetOverKill()
--			   -- Remove WOF FX trails if they exsist
--			   if self.WOF_FXTrailBag then
--				   EffectUtilities.CleanupEffectBag(self,'WOF_FXTrailBag')
--			   end
--			   -- Halt unit rotation if any
--			   if self.WOF_SpinManip_X and self.WOF_SpinManip_Y and self.WOF_SpinManip_Z then
--				   self.WOF_SpinManip_X:SetTargetSpeed(0)
--				   self.WOF_SpinManip_Y:SetTargetSpeed(0)
--				   self.WOF_SpinManip_Z:SetTargetSpeed(0)
--			   end
--			   -- When impacting ground during WOF event do deathweapon if avalible
--			   if new != 'Seabed' then
--				   if not self.WOF_DoDeathWeapon_Flag then
--					   -- WARN('OnLayerChange, DoDeathWeapon')
--					   self:DoDeathWeapon()
--				   end
--			   end
--			   if new == 'Land' or new == 'Seabed' then
--				  -- WARN('OnLayerChange, impacted', new)
--				   -- FX Cleanup
--				   self:DestroyAllDamageEffects()
--				   -- Death FX
--				   if self.PlayDestructionEffects then
--					   if self:GetCurrentLayer() == 'Water' then
--						   for k, v in EffectTemplate.TTorpedoHitUnitUnderwater01 do
--							 -- WARN('OnLayerChange, performing water destruction effects')
--							  CreateEmitterAtEntity( self, self:GetArmy(), v ):ScaleEmitter( self:GetFootPrintSize() * 2 )
--						   end
--					   else
--						  -- WARN('OnLayerChange, performing land destruction effects')
--						   self:CreateDestructionEffects( self, overkillRatio )
--					   end
--				   end
--				   self:PlayUnitSound('Destroyed')
--				  -- WARN('OnLayerChange, Overkill', overkillRatio)
--				   -- Create Corpse
--				   self:CreateWreckage( overkillRatio )
--				   -- Set damage effects based upon overkill
--				   if self.ShowUnitDestructionDebris then
--					   if overkillRatio <= 1 then
--						   self.CreateUnitDestructionDebris( self, true, true, false )
--					   elseif overkillRatio <= 2 then
--						  self.CreateUnitDestructionDebris( self, true, true, false )
--					   else
--						  self.CreateUnitDestructionDebris( self, true, true, true )
--					   end
--				   end
--				   -- Remove the unit from play
--				  -- WARN('OnLayerChange, Removing the unit from play')
--				   self:Destroy()
--				   return
--			   elseif new == 'Water' then
--				 --  WARN('OnLayerChange, impacted Water calling SinkThread')
--				   if old == 'Air' then
--					   for k, v in EffectTemplate.DefaultProjectileWaterImpact do
--						  CreateEmitterAtEntity( self, self:GetArmy(), v ):ScaleEmitter( self:GetFootPrintSize() )
--					   end
--				   end
--				   self:SinkThread( overkillRatio )
--			   end
--		   end
--		   oldUnit.OnLayerChange(self, new, old)
--	   end,
--
--	   SinkThread = function(self, overkillRatio)
--		   if self:IsDead() and not self:BeenDestroyed() and not self.WOF_SinkThread_Flag then
--			   -- Prevent SinkThread from running more then once
--			   self.WOF_SinkThread_Flag = true
--			   -- Determine the targets best bone to attach helper projectile
--			   local bone
--			   for i=-1, self:GetBoneCount() do
--				   if self:IsValidBone(i) then
--					   bone = i
--					   break
--				   end
--			   end
--			   --Ensure bone is avalible to pass
--			   if bone then
--				   -- Pass Data to the WOF Projectile
--				   local sink_data = {
--					   OverKill = overkillRatio,
--					   TargetBone = bone,
--					   TargetEntity = self,
--				   }
--				   -- Sinker WOF Projectile
--				   local proj = self:CreateProjectileAtBone('/mods/4DC_WOF_V0.8/projectiles/Sink_helper/Sink_helper_proj.bp', bone)
--				   proj.Data = sink_data
--			   else
--				   -- Remove the unit from play if the sinker is unable to run
--				   if not bone then
--					   WARN('SinkThread, "bone" not unavalible for unit ', self:GetUnitId())
--				   end
--				   if self:GetCurrentLayer() == 'Air' then
--					   explosion.CreateScalableUnitExplosion( self, overkillRatio )
--					   self:Destroy()
--				   else
--					   oldUnit.DeathThread( self, overkillRatio, instigator)
--					   return
--				   end
--			   end
--		   end
--	   end,
--
--	   CreateWreckage = function( self, overkillRatio )
--		   -- Ensure that CreateWreckage can run only once per unit
--		   if not self.WOF_CreateWreckage_Flag then
--			   self.WOF_CreateWreckage_Flag = true
--			   self:CreateWreckageProp(overkillRatio)
--		   end
--	   end,
--
--	   CreateWreckageProp = function( self, overkillRatio )
--		   -- Made a few minor changes to allow wreckage to spawn under water
--		   local bp = self:GetBlueprint()
--		   local wreck = bp.Wreckage.Blueprint
--		   if wreck then
--			   local pos = self:GetPosition()
--			   pos[2] = GetTerrainHeight(pos[1], pos[3]) + GetTerrainTypeOffset(pos[1], pos[3])
--			   local mass = bp.Economy.BuildCostMass * (bp.Wreckage.MassMult or 0)
--			   local energy = bp.Economy.BuildCostEnergy * (bp.Wreckage.EnergyMult or 0)
--			   local time = (bp.Wreckage.ReclaimTimeMultiplier or 1)
--			   local prop = CreateProp( pos, wreck )
--			   prop:AddBoundedProp(mass)
--			   prop:SetScale(bp.Display.UniformScale)
--			   prop:SetOrientation(self:GetOrientation(), true)
--			   prop:SetPropCollision('Box', bp.CollisionOffsetX, bp.CollisionOffsetY, bp.CollisionOffsetZ, bp.SizeX* 0.5, bp.SizeY* 0.5, bp.SizeZ * 0.5)
--			   prop:SetMaxReclaimValues(time, time, mass, energy)
--			   mass = (mass - (mass * (overkillRatio or 1))) * self:GetFractionComplete()
--			   energy = (energy - (energy * (overkillRatio or 1))) * self:GetFractionComplete()
--			   time = time - (time * (overkillRatio or 1))
--			   prop:SetReclaimValues(time, time, mass, energy)
--			   prop:SetMaxHealth(bp.Defense.Health)
--			   prop:SetHealth(self, bp.Defense.Health * (bp.Wreckage.HealthMult or 1))
--			   if not bp.Wreckage.UseCustomMesh then
--				   prop:SetMesh(bp.Display.MeshBlueprintWrecked)
--			   end
--			   TryCopyPose(self,prop,false)
--			   prop.AssociatedBP = self:GetBlueprint().BlueprintId
--			   -- Create FX based upon layer
--			   local surface = GetSurfaceHeight(pos[1], pos[3]) + GetTerrainTypeOffset(pos[1], pos[3])
--			   if self:GetCurrentLayer() == 'Water' then
--				   for i = 0, prop:GetBoneCount() do
--					   if prop:IsValidBone(i) then
--					   local propPos = prop:GetPosition(i)
--						   if propPos[2] < (surface - 5) then
--							   for k, v in Custom_WOF_EffectTemplate.AmbientSeabedEffects do
--								   -- Underwater bubbles
--								   CreateEmitterAtBone(prop, i, self:GetArmy(), v):ScaleEmitter( Random(1,2) )
--							   end
--						   elseif propPos[2] > surface then
--							   for k, v in EffectTemplate.DamageFireSmoke01 do
--								   -- Smoke / fire FX
--								   if table.find(self:GetBlueprint().Categories,'EXPERIMENTAL') then
--									   CreateEmitterAtBone(prop, i, self:GetArmy(), v):ScaleEmitter( Random(3, 4) )
--								   else
--									   CreateEmitterAtBone(prop, i, self:GetArmy(), v):ScaleEmitter( Random(0.25, 1) )
--								   end
--							   end
--						   end
--					   end
--				   end
--			   else
--				   -- Default FX
--				   explosion.CreateWreckageEffects(self,prop)
--			   end
--			   return prop
--		   else
--			   return nil
--		   end
--	   end,

}
end--(end of non-destructive hook)