------------------------------------------------------------------------------------------
--
--	File:		\mods\4DC_WOF_V0.8\hook\lua\system\Blueprints.lua
--
--	Author(s):	Resin_Smoker
--
--	Co-Auth:	Brute51, Domino
--
--	Updated:	11/03/2013
--
--	Summary :	Scripts added for Weight of Fire (W.O.F.) establish WOF restrictions, tech
--				level and add water impact audio to units that dont already have it.
--
--	Copyright  2013 WOF, All rights reserved.
--
------------------------------------------------------------------------------------------

do--(start of non-destructive hook)

	--Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
	local masterDebug = false

	--Hooking Blueprint.lua non-destructively
	local oldModBP = ModBlueprints
	function ModBlueprints(all_bps)-- Verified good 10/20/2013
	   oldModBP(all_bps)
	   
		--Cycle through all unit blueprints
		for id,bp in all_bps.Unit do
			if bp.Categories and table.getn(bp.Categories) > 0 then
			
				if masterDebug then
					LOG('*** BLUEPRINTS.LUA, ModBlueprints ***')
				end	  			
				--Use a copied table for testing cats.
				local cats = bp.Categories

				--Create the WOF initial settings within each units blueprint, this will later be passed into WOF_Stuff table duiring unit creation
				bp.WOF = {
					ALLOWED = true,								--Is unit allowed to have WOF events
					AVG_DIMENSION = 0.1,						--The units average dimensional size
					BONE_OFFSET = 0.0,							--The position offset the helper is to spawn to adjacent to the unit
					DESRIPTION = bp.Description or 'UNKNOWN',	--What am I
					IGNORE = false,								--The unit is neither allowed to have a WOF event nor has weapons
					NAME = bp.General.UnitName or 'UNKNOWN',	--My name is your name, lol
					TECH = 1,									
				}

				--Modify the above table by checking the specific cats for this unit
				WOF_CATS_CHECK(bp, cats)

				--Bypass if Ignore flag set "true" by WOF_CATS_CHECK
				if not bp.WOF.IGNORE then
					--Ensure that all units have an water impact sound assigned
					WOF_AUDIO_CHECK(bp, cats)
					--Add dimension properties to WOF tabel
					WOF_DIMENSIONAL(bp, cats)
					--Ensure that all units have some form of death impact
					WOF_IMPACT_CHECK(bp, cats)
					--Add tech level to WOF table to help calculate overkill between units of differing tech
					WOF_TECH_CHECK(bp, cats)
				end
				
				--Ensure that all units can create wreckage
				WOF_WRECKAGE_CHECK(bp, cats)				
			else
				WARN('*** Blueprints.lua, ModBlueprints, The following unit is with assigned categories: ' .. repr(bp.BlueprintId) or 'Unit ID unknown ***')		
			end

			if masterDebug then
				LOG('	WOF data set for this unit blueprint: ' ..repr(bp.WOF)or nil)
				if not bp.WOF.IGNORE then
					LOG('	ImpactWater sound for this unit blueprint: ' ..repr(bp.Audio.ImpactWater) or nil)
					for k, v in bp.Weapon do
						if v.DummyWeapon then
							LOG('	Dummy weapon: ' ..repr( v ) )				
						end
					end
					LOG('	bp.Wreckage for this unit: ' ..repr(bp.Wreckage) or nil )
				end
			end
		end
	end

	function WOF_CATS_CHECK(bp, cats)-- Verified good 10/20/2013	
		if masterDebug then
			LOG('	WOF_CATS_CHECK')
		end		

		--WOF "Unit" Restrictions
		local WOF_UNIT_RESTRICT = {
			-- Standard unit Restrictions
			'AIR','COMMAND','EXPERIMENTAL',
			'NAVAL','STRUCTURE','SUBCOMMANDER',
			-- Misc unit restrictions
			'BENIGN','INSIGNIFICANTUNIT',
			'OPERATION','POD','PROJECTILE',
			'SATELLITE','UNSELECTABLE','UNTARGETABLE',
			-- Custom unit restrictions
			'BLACKHOLE',
			'HELPER','HOLOGRAM','HOLOGENERATOR',
			'MINE','NOWOF',
		}

		--Verify the unit type is one that we are allowed to WOF,
		for k, v in cats do
			if table.find(WOF_UNIT_RESTRICT, v) then

				--Set flag to false, add identifier to units blueprint and return
				bp.WOF.ALLOWED = false
				table.insert (bp.Categories, 'NOWOF')
				break
			end
		end
				
		--Ignore unit types without weapons
		if not bp.Weapon and not bp.WOF.ALLOWED then
		
			--Set flag to false, add identifier to units blueprint and return
			bp.WOF.IGNORE = true
		end
	end

	function WOF_AUDIO_CHECK(bp, cats)-- Verified good 10/20/2013
		if masterDebug then
			LOG('	WOF_AUDIO_CHECK')
		end		
	
		--Generic WOF water impact
		local WOF_ImpactWater = Sound {
			Bank = 'Impacts',
			Cue = 'Impact_Water_Splash_AEON',
			LodCutoff = 'Weapon_LodCutoff',
		}

		-- Ensure that all mobile units all have a water impact sound
		if table.find(cats, 'MOBILE') and bp.WOF.ALLOWED == true then
			if not bp.Audio then
			
				--If a unit hasn't an Audio table, create one
				bp.Audio = {}
			end
			
			--Add ImpactWater to unit if not already defined				
			if not bp.Audio.ImpactWater then
				bp.Audio.ImpactWater = WOF_ImpactWater
			end
		end
	end
	
	function WOF_DIMENSIONAL(bp, cats)													
		if bp.WOF.ALLOWED then
		    --Calculate the units average dimensional size
		    bp.WOF.AVG_DIMENSION = math.abs( (bp.SizeX or 0 + bp.SizeY or 0 + bp.SizeZ or 0) / 3 ) 
		       					
			--The position offset the helper is to spawn to adjacent to the unit
			if bp.SizeY > 0 then
				bp.WOF.BONE_OFFSET = math.abs(bp.SizeY / 2)					
			end			
		end		
	end
	
	function WOF_IMPACT_CHECK(bp, cats)-- Verified good 10/20/2013
		if masterDebug then
			LOG('	WOF_IMPACT_CHECK')
		end		
		--If a unit hasn't an weapons table, create one
		if not bp.Weapon then
			bp.Weapon = {}
		end
		local ImpactWep = false
		for k, wep in bp.Weapon do
			if wep.DummyWeapon then
				--The weapon with this label is meant for aircraft and used when it hits the ground after being shot down.
				--For WOF hitting a surface may not be the the end of the unit, it can still sink, so changing it impact wep.
				if wep.Label == 'DeathImpact' then
					wep.Label = 'SurfaceImpact_Generic'
					ImpactWep = true
					break
				--Check if there is an impact weapon already defined
				elseif wep.Label == 'SurfaceImpact_Generic' or wep.Label == 'SurfaceImpact_Land' or wep.Label == 'SurfaceImpact_Water' or wep.Label == 'SurfaceImpact_Seabed' then
					ImpactWep = true
					break
				end
			end
		end
		if not ImpactWep then
			--Add the dummy weapon to the end of the weapons list
			local weap_num = table.getn(bp.Weapon) + 1
			
			--Get the max health possible for this unit
		    local dam = bp.Defense.MaxHealth or 1
		    	    
		    --Make radius at least 1 in all cases
		    local radius = math.max( 3.14 * bp.WOF.AVG_DIMENSION , 1 )
		    
		    --Add the dummy weapon
			bp.Weapon[weap_num] = {
				Damage = dam,
				DamageFriendly = true,
				DamageRadius = radius,
				DamageType = 'Normal',
				DisplayName = 'SurfaceImpact',
				DummyWeapon = true,
				FiringTolerance = 2,
				Label = 'SurfaceImpact_Generic',
				WeaponCategory = 'Impact',
			}
		end		
	end	

	function WOF_WRECKAGE_CHECK(bp, cats)-- Verified good 10/20/2013	
		if masterDebug then
			LOG('	WOF_WRECKAGE_CHECK')
		end
		
		--Calculate how long wreckage will live
	    local wreckage_time = bp.WOF.TECH * 300		
		
		--Add default wreckage if unit doesn't have a wreckage
		if not bp.Wreckage then		
			bp['Wreckage'] = {
				Blueprint = '/props/DefaultWreckage/DefaultWreckage_prop.bp',
				EnergyMult = 0,
				HealthMult = 0.9,
				MassMult = 0.9,
				ReclaimTimeMultiplier = 1,
				WreckageLayers = {
					Air = false,
					Land = true,
					Seabed = true,
					Sub = false,
					Water = false,
				},
			}
		end
		--Pass lifetime into Wreckage BP
		bp.Wreckage.Lifetime = wreckage_time								
	end

	function WOF_TECH_CHECK(bp, cats)-- Verified good 10/20/2013
		if masterDebug then
			LOG('	WOF_TECH_CHECK')
		end	

		--Add tech level to WOF table to help calculate overkill between units of differing tech
		for k, v in cats do
			if v == 'EXPERIMENTAL' then
				bp.WOF.TECH = 4
				return
			elseif v == 'SUBCOMMANDER' or v == 'TECH3' then
				bp.WOF.TECH = 3
				return
			elseif v == 'TECH2' then
				bp.WOF.TECH = 2
				return
			end
		end
	end

end--(end of non-destructive hook)