-----------------------------------------------------------------------------
--  File		/lua/WOF_PROJ_UTILS.lua 
-- 
--  Author		Resin_Smoker 
-- 
--  Co-Auth	    Brute51, Domino
--
--  Updated	    10/20/2013
--
--  Summary		 Allows WOF and sink effects to take place
-- 
--  Copyright  2011 4DC_V0.81  All rights reserved.
--
--  Notes		Bone 0 = 'wof_helper',  Bone 1 = 'anchor'  
--
-----------------------------------------------------------------------------
-- Misc local files
local utilities = import('/lua/utilities.lua')
local EffectTemplate = import('/lua/EffectTemplates.lua')
local EffectUtil = import('/lua/EffectUtilities.lua') 
local explosion = import('/lua/defaultexplosions.lua')
local RandomFloat = import('/lua/utilities.lua').GetRandomFloat

-- Custom files imported
local WOF_UNIT_UTILS = import('/mods/4DC_WOF_V0.8/lua/WOF_UNIT_UTILS.lua')

-- Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
local masterDebug = true

do

    WOF_START = function( proj, unit )
		local myDebug = false
		if myDebug or masterDebug then
			LOG('*** WOF_PROJ_UTILS, WOF_LAUNCH ***')					
		end				 
		if proj and not proj:BeenDestroyed() then						
	    	if unit and not unit:BeenDestroyed() then
				local data = unit.WOF_STUFF	    	
				local proj_bone = 'anchor'	    	
		    	local unit_army = data.UNIT.ARMY or unit:GetArmy()
				local unit_avg_size = data.UNIT.AVG_DIMENSION or '0'		    	
				local unit_bone = data.UNIT.BONE or '0' 
				local unit_bone_offset = data.UNIT.BONE_OFFSET or '0'
				local vX, vY, vZ = unpack(data.UNIT.DMG_VECTOR or data.UNIT.IMPACT_VECTOR)
		    	WOF_ATTACH_TO_ANCHOR( proj, proj_bone, unit, unit_bone, unit_bone_offset )
		    	ForkThread( WOF_FX_TRAILS, proj, unit, unit_army, unit_bone, unit_avg_size )
		    	local vectors = WOF_CALC_VECTORS(unit, data)
		    	WOF_LAUNCH( proj, unit, vectors )
			else
				if myDebug or masterDebug then
					LOG('*** WOF_PROJ_UTILS, WOF_LAUNCH ***')
					LOG('	unit is missing or destroyed !!!') 
				end
		    	proj:Destroy()
		    end
		elseif myDebug or masterDebug then
				LOG('*** WOF_PROJ_UTILS, WOF_LAUNCH ***')
				LOG('	proj is missing or destroyed !!!') 	
		end     	
    end
        
	function WOF_CALC_VECTORS( unit, data )    
		ProjVector = data.UNIT.DMG_VECTOR or data.UNIT.IMPACT_VECTOR   

        --Calc angle from given vector
        local vx = ProjVector[1]
        local vz = ProjVector[3]
        local ProjAngle = math.acos( vx / math.sqrt( math.pow(vx, 2) + math.pow(vz, 2) ) )
        LOG('proj angle = '..repr(ProjAngle))

        --Get unit movement angle
        local MoveAngle = unit:GetHeading() * 180 / math.pi
        --LOG('MoveAngle  = '..repr(MoveAngle))

        --Determine median: angle between proj impact and movement
        local median = ( MoveAngle - math.abs(ProjAngle) ) / 2

        --Calculate launch angle, factor in the unit speed
        local speedFactor = data.UNIT.MOVEMENT_FACTOR or 0
        LOG('speedFactor = '..repr(speedFactor))
        local angle = ProjAngle + (median * speedFactor)
        LOG('angle = '..repr(angle))
        return angle
    end    
    
	function WOF_LAUNCH( proj, vectors )
        -- Launch the helper on the vectors supplied                                                                    
        proj:SetVelocity( unpack( vectors ) )  	
	end    
       
	function WOF_ATTACH_TO_ANCHOR(  proj, proj_bone, unit, unit_bone, unit_bone_offset )
		local myDebug = false
		if myDebug or masterDebug then
			LOG('*** WOF_PROJ_UTILS.lua, WOF_FX_TRAILS ***')
		end
		local x, y, z = unpack( unit:GetPosition( ) )
		if unit_bone_offset > 0 then
			vY = unit_bone_offset
			--Move proj to the middle of where the unit would be
			Warp( proj, Vector(x,vY,z) ) 			
		end
							
		--Attach the target unit to the anchor bone
		unit:AttachBoneTo( unit_bone, proj, proj_bone )
		if unit_bone_offset > 0 then
			Warp( unit, Vector(x,y,z) ) 
		end
    end

    function WOF_FX_TRAILS( proj, unit, unit_army, unit_bone, unit_avg_size  )
		local myDebug = false
		if myDebug or masterDebug then
			LOG('*** WOF_PROJ_UTILS.lua, WOF_FX_TRAILS ***')	
		end
		if proj and unit and unit_army and unit_avg_size then			        
	        -- Add FX trails based on faction
	        if table.find(unit:GetBlueprint().Categories,'AEON') then 
	            table.insert(unit.WOF_STUFF.FX, CreateAttachedEmitter( unit, unit_bone, unit_army, '/effects/emitters/miasma_munition_trail_01_emit.bp'):ScaleEmitter( unit_avg_size * 10 ) )
	        elseif table.find(unit:GetBlueprint().Categories,'CYBRAN') then 
	            table.insert(unit.WOF_STUFF.FX, CreateAttachedEmitter( unit, unit_bone, unit_army, '/effects/emitters/destruction_damaged_smoke_01_emit.bp'):ScaleEmitter( unit_avg_size * 10 ) ) 
	        elseif table.find(unit:GetBlueprint().Categories,'SERAPHIM') then
	            table.insert(unit.WOF_STUFF.FX, CreateAttachedEmitter( unit, unit_bone, unit_army, '/effects/emitters/seraphim_rift_gateway_01_emit.bp'):ScaleEmitter( unit_avg_size * 10 ) ) 
	        else
	            table.insert(unit.WOF_STUFF.FX, CreateAttachedEmitter( unit, unit_bone, unit_army, '/effects/emitters/destruction_explosion_fire_plume_01_emit.bp'):ScaleEmitter( unit_avg_size * 10 ) ) 
	        end
		elseif myDebug or masterDebug then
			LOG('	Some data did not pass')			
		end         
	end
	

	   
--     WOF_LAUNCH_OLD = function(self)
--         -- Ensure the data has passed and that we have a target 
--         if not self.Data then 
--             LOG('*** WOF Helper, Data either failed to pass or target no longer exists ***')
--             LOG('	', repr(self))           
--             self:Destroy()
--             return
--         end          
-- 	                            
-- 	        -- Calculate vectors based upon the targets current layer
-- 	        local X, Y, Z = unpack(self.Data.UNIT.DMG_VECTOR)
-- 
-- 	        
-- 	        
--         local vX, vY, vZ         
--         local layer = target:GetCurrentLayer()
--         if layer == 'Air' then        -- x / ( abs( x)  + 1 )
--             -- Done if a unit was killed while on a transport of some kind
--             vX = ratio * X
--             vY = -0.1
--             vZ = ratio * Z                               
--         else 
--             vX = (ratio * X) 
--             vY = ratio / ( math.abs(Y)  + 1 ) -- (ratio * math.abs(Y)) + RandomFloat(-1.00, 1.00)  -- ((x - 10) / ( abs( x - 10)  + 2 ) * 0.525) + 0.44       base formula x / (x + 1)
--             vZ = (ratio * Z)   
--         end   
--         -- Launch the helper on the vectors supplied                                                                    
--         self:SetVelocity( vX, vY, vZ)                                                     
--         -- Create ground / water FX on the unit impacted
--         if layer == 'Air' then
--             explosion.CreateUnitExplosionEntity( target, ratio)
--             explosion.CreateDebrisProjectiles(target, explosion.GetAverageBoundingXYZRadius(target), {target:GetUnitSizes()})                
--         else
--             if layer == 'Land' then  
--                 for k, v in EffectTemplate.THeavyFragmentationGrenadeHit do 
--                     if IsUnit(target) then
--                         CreateEmitterAtEntity( target, target:GetArmy(), v ):ScaleEmitter( explosion.GetUnitVolume(target) + 1  )
--                     end
--                 end  
--             elseif layer == 'Water' then 
--                 for k, v in EffectTemplate.TIFCruiseMissileLaunchExitWater do 
--                     if IsUnit(target) then
--                         CreateEmitterAtEntity( target, target:GetArmy(), v ):ScaleEmitter( explosion.GetUnitVolume(target) + 1  )
--                     end
--                 end 
--             end 
--         end
--                                                    
--         -- Attach the target to the helper projectile
--         target:AttachBoneTo(self.Data.TargetBone, self, 'anchor')      
--         -- Set IsAttached_Flag
--         target.IsAttached_Flag = true                                                                               
--         -- Spin the target wildly       
--         target.WOF_SpinManip_X = CreateRotator(target, 0, 'x', nil, 0, X * 100, 1000) 
--         target.Trash:Add(target.WOF_SpinManip_X)
--         target.WOF_SpinManip_Y = CreateRotator(target, 0, 'y', nil, 0, Y * 100, 1000)              
--         target.Trash:Add(target.WOF_SpinManip_Y)  
--         target.WOF_SpinManip_Z = CreateRotator(target, 0, 'z', nil, 0, Z * 100, 1000)
--         target.Trash:Add(target.WOF_SpinManip_Z)                                                              
--     end,
--     
--     OnDestroy = function(self)
--         if not self.Data.TargetEntity:BeenDestroyed() then 
--             local target = self.Data.TargetEntity       
--             target:DetachFrom(false)
--             target.IsAttached_Flag = false
--             target.WOF_Active_Flag = false
--         end     
--         SuperClass.OnDestroy(self)
--     end,
--            
--     OnImpact = function(self, targetType, targetEntity)         
--         self:Destroy()                                      
--     end,      
           
end--(end of non-destructive hook)