------------------------------------------------------------------------------------------
--
--  File     :  \mods\4DC_WOF_V0.8\hook\lua\WOF_UNIT_UTILS.lua
--
--  Author(s):  Resin_Smoker
--
--  Co-Auth:    Brute51, Domino
--
--  Updated:    10/13/2013
--
--  Summary  :  Scripts added for Weight of Fire (W.O.F.) for misc functions
--
--  Copyright  2013 4DC_WOF_V0.8 WOF, All rights reserved.
--
------------------------------------------------------------------------------------------
do--(start of non-destructive hook)

    -- Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
    local masterDebug = false
    
    -- Large numbers of wreckages cause the game to slowdown. This mod adds more wreckages than usual. Set this to true to remove wreckages on the seabed if left alone for too long.
    AllowWreckageDecay = true    
    
    -- FA misc files imported
	local explosion = import('/lua/defaultexplosions.lua')
	local EffectTemplate = import('/lua/EffectTemplates.lua')
	local utilities = import('/lua/utilities.lua')
	local GetRandomFloat = import('/lua/utilities.lua').GetRandomFloat			     

	-- Custom files imported    
	local WOF_PROJ_UTILS = import('/mods/4DC_WOF_V0.8/lua/WOF_PROJ_UTILS.lua')
	local Custom_WOF_EffectTemplate = import('/mods/4DC_WOF_V0.8/lua/WOF_EffectTemplates.lua')	
			      
# =======================================================================================================================
# WOF UNIT.LUA 
# =======================================================================================================================     
    
    function WOF_ON_CREATE(self, bp)-- Verified good 10/16/2013
		local myDebug = false
		if self and bp then	
			self.WOF_STUFF = {}                  							--Prime WOF_STUFF Global
			self.WOF_STUFF.FX = {}											--Prime effects table
			self.WOF_STUFF.INST = {}                  						--Prime Instigator table
			self.WOF_STUFF.PROJ = {}                  						--Prime Projectile table
			self.WOF_STUFF.TRASH = TrashBag()								--Establish trashbag for effects				
			self.WOF_STUFF.WEAP = {}                  						--Prime Weapon table				
			self.WOF_STUFF.UNIT = bp.WOF									--Copy BP data to sim global
			self.WOF_STUFF.UNIT.ACTIVE = false								--Unit flag Indicates if WOF ongoing
			self.WOF_STUFF.UNIT.ARMY = self:GetArmy()						--Copy Army data
			self.WOF_STUFF.UNIT.BP = self:GetBlueprint()					--Make a copy of the units BP for later use								
			self.WOF_STUFF.UNIT.BONE = WOF_GET_BONE( self ) 				--Units assigned bone to connect with helper projectile	
			self.WOF_STUFF.UNIT.DEATH_ANIMATION = false						--Unit flag indicates that death animation has triggered once already				
			self.WOF_STUFF.UNIT.DEATH_THREAD = false						--Unit flag indicates that death thread has triggered once already
			self.WOF_STUFF.UNIT.DEATH_WEAPON = false						--Unit flag indicates that death weapon has triggered once already						
			self.WOF_STUFF.UNIT.DMG_VECTOR = {}								--Direction / side unit was impacted		
			self.WOF_STUFF.UNIT.HELPER_PROJ = {}							--Units current helper projectile
			self.WOF_STUFF.UNIT.ID = self:GetEntityId() 					--Units ID tag					
			self.WOF_STUFF.UNIT.IN_TRANSPORT = false						--Units transport status
			self.WOF_STUFF.UNIT.LAYER = self:GetCurrentLayer() or 'None'	--Units current layer
			self.WOF_STUFF.UNIT.ON_KILLED = false							--Unit flag indicates the OnKilled thread has triggered once already
			self.WOF_STUFF.UNIT.MOVEMENT_FACTOR = {}						--Current rate of movement
			self.WOF_STUFF.UNIT.OVERKILL = 0.0								--Units amount of damage overkill
			self.WOF_STUFF.UNIT.READY_TO_LAUNCH = false						--Unit flag Indicates WOF eval is good and that unit is now ready to launch				
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('*** WOF_UNIT_UTILS.lua, WOF_ONCREATE ***')		
				LOG('	WOF_STUFF: '.. repr(self.WOF_STUFF) )
			end							
		elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** WOF_UNIT_UTILS.lua, WOF_ONCREATE ***')
			LOG('	Data not avalible for this unit')
			WOF_ID_MY_UNIT( self )
		end    
    end
    
    function WOF_ON_DESTROY(self)-- Verified good 10/20/2013
		local myDebug = false    
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** WOF_UNIT_UTILS.lua, WOF_ON_DESTROY ***')		
		end	     
        --Destroy trashbags
        if self.WOF_STUFF.TRASH and not self.WOF_STUFF.TRASH == nil then
            self.WOF_STUFF.TRASH:Destroy()
        end

        --Destroy helper projectile if existing
        if self.WOF_STUFF.UNIT.HELPER_PROJ and not self.WOF_STUFF.UNIT.HELPER_PROJ == nil then
            self.WOF_STUFF.UNIT.HELPER_PROJ:Destroy()
        end    
    end
    
    function WOF_ON_MOTION_EVENT_CHANGE( self, new, old )
        --Need to know the speed at which the unit is traveling in case it is destroyed and we're gonna launch the unit under
        --WOF effect. These are just arbitrary values. Cruising is probably done when in formation with other slower units.
        if not self:IsDead() and new != old then
                if new == 'Stopped'  then self.WOF_STUFF.UNIT.MOVEMENT_FACTOR = 0
            elseif new == 'Stopping' then self.WOF_STUFF.UNIT.MOVEMENT_FACTOR = 0.25
            elseif new == 'Cruise'   then self.WOF_STUFF.UNIT.MOVEMENT_FACTOR = 0.75
            elseif new == 'TopSpeed' then self.WOF_STUFF.UNIT.MOVEMENT_FACTOR = 1
            else WARN('OnMotionHorzEventChange: unhandled speed event: '..repr(new)) end
        end
    end
    
    function WOF_ON_DAMAGE( self, instigator, amount, vector, damageType )-- Verified good 10/16/2013
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** WOF_UNIT_UTILS.lua, WOF_ON_DAMAGE ***')								
		end
		if not self.WOF_STUFF.UNIT.IGNORE and self.WOF_STUFF.UNIT.ALLOWED and amount > 0 and amount > self:GetHealth() and ( damageType != 'Reclaimed' ) then			
			--Insert damage vector into WOF_STUFF
			self.WOF_STUFF.UNIT.DMG_VECTOR = vector
			
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	self.WOF_STUFF repr: ', repr( self.WOF_STUFF ))								
			end										
			--Verify that the instigator data is carried from the projectile to the units WOF_STUFF table						
			if not self.WOF_STUFF.INST then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "WOF_STUFF.INST" doesnt exist, fixing it')
				end				
				self.WOF_STUFF.INST = {}
			end			
			if self.WOF_STUFF.INST.ID == nil or self.WOF_STUFF.INST.ID == 'UNKNOWN' then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "instigator" is bad, fixing it')
				end			
				if IsUnit(instigator) and instigator != nil then
					self.WOF_STUFF.INST.ID = instigator:GetEntityId()
				else
					self.WOF_STUFF.INST.ID = 'UNKNOWN'
				end
			elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	WOF_ON_DAMAGE, "instigator" is good')	
			end
			--Verify that the TECH data is carried from the projectile to the units WOF_STUFF table			
			if not self.WOF_STUFF.INST.TECH then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "TECH" is bad, fixing it')
				end			
				if IsUnit(instigator) and instigator != nil then
					self.WOF_STUFF.INST.TECH = instigator:GetBlueprint().WOF.TECH 
				else
					self.WOF_STUFF.INST.TECH = 1 --(default tech level if not known)
				end			
			elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	WOF_ON_DAMAGE, "TECH" is good')		
			end			
			--Verify that the damage data is carried from the projectile 					
			if not self.WOF_STUFF.PROJ then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "WOF_STUFF.PROJ" doesnt exist, fixing it')
				end						
				self.WOF_STUFF.PROJ = {}
			end
			--Verify that the damage amount is carried from the projectile 			
			if amount != self.WOF_STUFF.PROJ.DMG_AMOUNT then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "amount" is bad, fixing it')
				end				
				self.WOF_STUFF.PROJ.DMG_AMOUNT = amount or 0
			elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	WOF_ON_DAMAGE, "amount" is good')	
			end
			--Verify that the damage type is carried from the projectile 			
			if damageType != self.WOF_STUFF.PROJ.DMG_TYPE then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	WOF_ON_DAMAGE, "damageType" is bad, fixing it')
				end					
				self.WOF_STUFF.PROJ.DMG_TYPE = damageType or 'UNKNOWN'
			elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	WOF_ON_DAMAGE, "damageType" is good')		
			end						
			--Evaluate the unit for WOF launch					
			WOF_EVALUATE( self, instigator, amount, vector, damageType )
			return --Return to Unit.lua OnDamage			
		elseif myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('	Conditions do not meet the requiements for WOF evaluation, defaulting to oldUnit.OnDamage')
			return --Return to unit.lua OnDamage	
		end    
    end

	function DO_TAKE_DAMAGE(self, instigator, amount, vector, damageType)
        local myDebug = false
        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, DO_TAKE_DAMAGE ***')
        end	        
		--If our unit is a transport, randomly detach cargo each time we are hit
		if self and EntityCategoryContains( categories.TRANSPORTATION, self ) then
			local cargo = self:GetCargo()
			local num = table.getsize(cargo)
			if num > 0 then
				local ran = Random(1, num)
				cargo[ran]:DetachFrom(true)
			end
		end
		
		--Detach a unit in transport that takes damage.
        if self.WOF_STUFF.UNIT.IN_TRANSPORT then
            self:DetachFrom(true)                        
        end		 
	end
	
	function ON_KILLED( self, instigator, type, overkillRatio )
		local myDebug = false
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('*** WOF_UNIT_UTILS.lua, ON_KILLED ***')
		end	
		-- Ensure that OnKilled can run only once per unit
		if not self.WOF_STUFF.UNIT.ON_KILLED then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Setting WOF_STUFF.UNIT.ON_KILLED flag to prevent OnKilled from running more than once')
			end		
			--Set flag to true so ON_KILLED event will not run more than once
			self.WOF_STUFF.UNIT.ON_KILLED = true
			return true
		else
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Attempted to run a second time for unit ')
			end
			return false
		end
	end
	
	function DO_DEATH_WEAPON( self, overkillRatio )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('	WOF_UNIT_UTILS.lua, DO_DEATH_WEAPON')						
		end		
		-- Bypass death weapon if still being built or WOF triggered				
		if not self:IsBeingBuilt() or not WOF_UNIT.READY_TO_LAUNCH or not self.WOF_STUFF.UNIT.ACTIVE and not self.WOF_STUFF.UNIT.DEATH_WEAPON then
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Run Unit.lua, DoDeathWeapon')						
			end
			--Set flag to true so DEATH_WEAPON event will not run more than once
			self.WOF_STUFF.UNIT.DEATH_WEAPON = true	 
			return true
		else
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Bypass Unit.lua, DoDeathWeapon')						
			end		 		
			return false
		end		
	end
	
	function PLAY_ANIMATION_THREAD( self, anim, rate )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('	WOF_UNIT_UTILS.lua, PLAY_ANIMATION_THREAD')						
		end	
		-- Bypass death animations if WOF triggered
		if not self:BeenDestroyed() and not self.WOF_STUFF.UNIT.ACTIVE then
			-- Ensure that PlayAnimationThread can run only once per unit during non WOF event
			if not self.WOF_STUFF.UNIT.DEATH_ANIMATION then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then			
					LOG('	Return to Unit.lua PlayAnimationThread')
				end				
				self.WOF_STUFF.UNIT.DEATH_ANIMATION = true
				return true 
			else
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then			
					LOG('	Death animation attempted to run mutlible times, bypass Unit.lua PlayAnimationThread')
				end
				return false
			end
		else
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then			
				LOG('	Unit destroyed or WOF active, bypass Unit.lua PlayAnimationThread')
			end			
			return false
		end	
	end
	
	function DEATH_THREAD( self, overkillRatio, instigator )
		local myDebug = false 
		if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
			LOG('	WOF_UNIT_UTILS.lua, DEATH_THREAD')						
		end
		if not self.WOF_STUFF.UNIT.DEATH_THREAD then
			--Set flag to keep DeathThread from running more than once
			self.WOF_STUFF.UNIT.DEATH_THREAD = true
			if not self:BeenDestroyed() and self.WOF_STUFF.UNIT.ALLOWED and not self.WOF_STUFF.UNIT.ACTIVE and self.WOF_STUFF.UNIT.READY_TO_LAUNCH then
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	Bypassing old DeathThread to launching this unit')						
				end			 
				WOF_LAUNCH( self )
				return false --Return to Unit.lua DeathThread and skip death event
			else
				if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
					LOG('	Running old DeathThread')						
				end				
				return true --Return to Unit.lua DeathThread to run old thread
			end
		else
			if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
				LOG('	Bypassing launch and old DeathThread, unit attempted to run DeathThread more than once')						
			end			
			return false --Return to Unit.lua DeathThread and skip death event
		end
	end
	
    function ON_LAYER_CHANGE( self, new, old )-- Verified good 10/16/2013
        local myDebug = false
        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, ON_LAYER_CHANGE ***')           
        end    
    	if not self:BeenDestroyed() and self.WOF_STUFF then
			if new != 'None' and new != old then
				self.WOF_STUFF.UNIT.LAYER = new
		        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
		            LOG('	', self.WOF_STUFF.UNIT.LAYER)
		        end 					
			end
		end                                     
    end
    
# =======================================================================================================================
# Unit Wreckage
# =======================================================================================================================     
	local wreckdebug = false    

	function WOF_CREATE_WRECKAGE_PROP( self, overkillRatio )
        local myDebug = true
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_CREATE_WRECKAGE_PROP ***')           
        end
        --Grouping all of the locals used for simplicity sake and to prevent defining them multible times across the individual threads.
        local bp = self:GetBlueprint()
		local pos = self:GetPosition()
		local size = utilities.GetRandomFloat( 0.5, math.max( ( self.WOF_STUFF.UNIT.AVG_DIMENSION or 1 ), 2) )		
		--Spawn the wreckage prop
        local wreck = WOF_SPAWN_WRECK_PROP( self, overkillRatio, pos, bp )
        if wreck and IsProp(wreck) then
	        --Define the wrecks reclaim value
	        WOF_SET_RECLAIM( self, overkillRatio, wreck, bp )
			--If we're adding effects we need an effects bag to prevent memory leaks. Wreckages don't have one so add one here.        		         
	        WOF_WRECKAGE_TRASH( self, wreck )
			--Wreckage decay. We dont decay wreckages created in the first 10 game seconds, these are normally map specific.        
			WOF_WRECKAGE_DECAY( self, wreck )
			--Scaleable wreckage FX           							
			WOF_SCALEABLE_WRECKAGE_FX( self, size, wreck, self.WOF_STUFF.UNIT.ARMY )	
			return wreck
		else
			wreck:Destroy()
		end
	end
	
	function WOF_SPAWN_WRECK_PROP( self, overkillRatio, pos, bp )
        local myDebug = false
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_CREATE_WRECK_PROP ***')           
        end 	
		-- Made a few minor changes to allow wreckage to spawn under water
		if bp and bp.Wreckage.Blueprint then
			local x,y,z = unpack( pos )
			y = GetTerrainHeight(x, z) + GetTerrainTypeOffset(x, z)
			local wreck = CreateProp( Vector(x,y,z) , bp.Wreckage.Blueprint )					
			wreck:SetScale( bp.Display.UniformScale )
			wreck:SetOrientation( self:GetOrientation(), true )
			wreck:SetPropCollision( 'Box', bp.CollisionOffsetX, bp.CollisionOffsetY, bp.CollisionOffsetZ, bp.SizeX* 0.25, bp.SizeY* 0.25, bp.SizeZ * 0.25 )
			if not bp.Wreckage.UseCustomMesh then
				wreck:SetMesh( bp.Display.MeshBlueprintWrecked )
			end
			TryCopyPose( self, wreck, false )
			wreck.AssociatedBP = self:GetBlueprint().BlueprintId
			return wreck
		end
	end
	
	function WOF_SET_RECLAIM( self, overkillRatio, wreck, bp)
        local myDebug = false
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_SET_RECLAIM ***')          
        end     	
		local mass = bp.Economy.BuildCostMass * ( bp.Wreckage.MassMult or 0 )
		local energy = bp.Economy.BuildCostEnergy * ( bp.Wreckage.EnergyMult or 0 )
		local time = ( bp.Wreckage.ReclaimTimeMultiplier or 1 )
		wreck:AddBoundedProp( mass )
		wreck:SetMaxReclaimValues( time, time, mass, energy )
		mass = ( mass - ( mass * ( overkillRatio or 1 ) ) ) * self:GetFractionComplete()
		energy = ( energy - ( energy * ( overkillRatio or 1 ) ) ) * self:GetFractionComplete()
		time = time - ( time * ( overkillRatio or 1 ) )	
		wreck:SetReclaimValues( time, time, mass, energy )
		wreck:SetMaxHealth( bp.Defense.Health )
		wreck:SetHealth( self, bp.Defense.Health * (bp.Wreckage.HealthMult or 1) )						
	end	
	
	function WOF_WRECKAGE_TRASH( self, wreck )
        local myDebug = false
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_WRECKAGE_TRASH ***')          
        end          	        
        if not wreck.Trash then              	
            wreck.Trash = TrashBag()
            wreck.oldOnDestroy = wreck_old.OnDestroy
            wreck.OnDestroy = function(self)
                if self.Trash then
                    self.Trash:Destroy()
                end
                self.oldOnDestroy( self )
            end
        end
	end		
	
	function WOF_WRECKAGE_DECAY( self, wreck )
        local myDebug = false
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_WRECKAGE_DECAY ***')           
        end   	
		if AllowWreckageDecay then                      
		    if GetGameTimeSeconds() > 10 then
		    	--Lifetime set by Blueprints.lua and is based on Tech level
		        local time = self.WOF_STUFF.UNIT.BP.Wreckage.Lifetime or 60                
		        wreck.DecayFn = function(self, time )
		            WaitSeconds( time )
		            if self and not self:BeenDestroyed() then
		                self:Destroy()
				        if myDebug or masterDebug then
				            LOG('	Wreck Destroyed after a delay of: ', time)           
				        end                                
		            end
		        end
		        wreck.Trash:Add( ForkThread( wreck.DecayFn,wreck, time ) )
		    end
		end
	end
		
	function WOF_SCALEABLE_WRECKAGE_FX( self, size, wreck, army )
        local myDebug = true
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_SCALEABLE_WRECKAGE_FX ***')           
        end
        
        --Above water effects
        local effect_aw1 = EffectTemplate.DamageFireSmoke01
        local effect_aw2 = Custom_WOF_EffectTemplate.AboveWaterDamageEffect1
        local effect_aw3 = Custom_WOF_EffectTemplate.AboveWaterDamageEffect2
        --Under water effects
        local effect_uw1 = Custom_WOF_EffectTemplate.AmbientSeabedEffects
        local effect_uw2 = Custom_WOF_EffectTemplate.UnderWaterDamageEffect1
        local effect_uw3 = Custom_WOF_EffectTemplate.UnderWaterDamageEffect2 
        
        local GetBone, NumBones = GetRandomBoneFn( wreck )          
        --Adjust number of emitters for unit size, an experimental unit gets more than a small scout
        local NumEmitters = NumBones * math.max( 0.25, explosion.GetAverageBoundingXYZRadius( wreck ) )
        
        for i=0, NumEmitters do
            local bone = GetBone()
            if bone then
                bone = bone - 1      -- a correction, somehow an invalid bone is sometimes returned which gives a warning
            else
                continue
            end

            local pos = wreck:GetPosition( bone )
        	local surface_height = GetSurfaceHeight(pos[1],pos[2]) + GetTerrainTypeOffset(pos[1],pos[2])
        	local emit = {}
            local ran = Random(1, 3)        	
            if pos[2] >= surface_height then
                --Create fire or sparks only if emitters if bone is above water line  
				if ran == 1 then
              		--Fire	
                    emit = WOF_SCALEABLE_EMITTERS( wreck, effect_aw1, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army)
                elseif ran == 2 then
                	--Spark if emitters if bone is above water line                      
                     emit = WOF_SCALEABLE_EMITTERS( wreck, effect_aw2, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army)
                else
                	--Spark                                               
                    emit = WOF_SCALEABLE_EMITTERS( wreck, effect_aw3, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army)
                end
            elseif pos[2] < surface_height then
                --Create air bubbles or sparks if bone is below water line and only if enough under water              

                if ran == 1 and pos[2] <= (surface_height - 5) then
               		--Bubbles
                    emit = WOF_SCALEABLE_EMITTERS( wreck, effect_uw1, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army) 
                elseif ran == 2 then 
                	--Spark                   	
                    emit = WOF_SCALEABLE_EMITTERS( wreck, effect_uw2, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army)  
                else
                   	--Spark                
                    emit = WOF_SCALEABLE_EMITTERS( wreck, effect_uw3, bone, nil, nil, nil, GetRandomFloat(0.25, size), GetRandomFloat( 60, 300 ), army)
                end
            end
            table.insert( wreck.Trash , emit )
        end     
	end

    
# =======================================================================================================================
# WOF SPECIFIC STUFF
# =======================================================================================================================              
	function WOF_SCALEABLE_EMITTERS( unit, effect, offsetX, offsetY, offsetZ, scale, lifetime, army )
        local myDebug = true
        if myDebug or wreckdebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_SCALEABLE_EMITTERS ***')           
        end	
	
	    local emitters = {}
	    for k, v in effect do
	        local emit = CreateEmitterAtEntity( unit, army or unit:GetArmy(), v ):ScaleEmitter( scale or explosion.GetAverageBoundingXYZRadius(unit) ) 
	        if offsetX and offsetY and offsetZ then
	            emit:OffsetEmitter( offsetX, offsetY, offsetZ)
	        end
	        if lifetime and lifetime > 0 then
	            emit:SetEmitterParam( 'LIFETIME', lifetime )
	        end
	        table.insert( emitters, emit )
	    end
	    return emitters
	end

    function WOF_GET_BONE(unit)-- Verified good 10/16/2013
        -- Determine the targets best bone to attach helper projectile
        local myDebug = false
        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS, WOF_GET_BONE ***')
            WOF_ID_MY_UNIT( unit )
        end
        for i=-1, unit:GetBoneCount() do
            if unit:IsValidBone(i) then
                if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
                    LOG('    Valid Bone: ', i)
                end
                return i
            end
        end
        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('   Could not determine bone for WOF effect on unit: ', unit:GetUnitId())
        end
    end

	function GetRandomBoneFn( unit )
	    --Returns a function that, when called, returns a random bone in unit. Returns each bone only once.
	    local bones = {}
	    local m = unit:GetBoneCount()
	    for bone=-1, m do
	        if unit:IsValidBone( bone ) then
	            table.insert( bones, bone )
	        end
	    end
	    return RandomIter( bones ), table.getsize( bones )
	end

    function WOF_PASS_PROJ_DATA_TO_UNIT (proj, damageData, targetEntity)-- Verified good 9/26/2013
    	--Passes all signifigant data from the projectile to the impacted units WOF_STUFF
        local myDebug = false
        if myDebug or masterDebug then
            LOG('*** WOF_UNIT_UTILS, WOF_PASS_PROJ_DATA_TO_UNIT ***')
            WOF_ID_MY_UNIT( targetEntity )            
        end
        --End function if proj is missing        
        if not proj then
            if myDebug or masterDebug then
                LOG('   "proj" is missing')
            end
            return
        end          
        --End function if damageData is missing              
        if not damageData then        
            if myDebug or masterDebug then
                LOG('   "damageData" is missing')
            end
            return
        end
        --End function if targetEntity is missing         
        if not targetEntity then       
            if myDebug or masterDebug then
                LOG('   "targetEntity" is missing')
            end
            return
        end             
        -- Insert the instigator into damageData if it was not avalible from weapon.lua GetDamageTable
        if damageData.INST.ID == nil then
        	damageData.INST.ID = proj:GetLauncher():GetEntityId() or targetEntity:GetEntityId() or 'NIL'         
	        if myDebug or masterDebug then
	            LOG('   Instigator missing, Passed info to damageData')
	            LOG('    '.. damageData.INST.ID )
	        end        
        end        
        --Insert proj vectors into damageData
        damageData.PROJ.IMPACT_VECTOR = Vector(proj:GetVelocity())
        if myDebug or masterDebug then
            LOG('   Proj vector Passed to damageData')
            LOG('    '.. repr( damageData.PROJ.IMPACT_VECTOR ) )
        end          
        -- Pass the signifigant data to the targetEntity WOF_STUFF table    
        targetEntity.WOF_STUFF.INST = {}
        targetEntity.WOF_STUFF.INST = damageData.INST
        targetEntity.WOF_STUFF.PROJ = {}        
        targetEntity.WOF_STUFF.PROJ = damageData.PROJ
        targetEntity.WOF_STUFF.WEAP = {}          
        targetEntity.WOF_STUFF.WEAP = damageData.WEAP
        if myDebug or masterDebug then
            LOG('   Projectiles Data Passed to targetEntity:')       
            LOG('   '.. repr( targetEntity.WOF_STUFF ) )
        end           
    end		    
            
    function WOF_EVALUATE( self, instigator, amount, vector, damageType )
    	--Compairs the level of overkill with the attacker and defenders tech levels
        local myDebug = false
        if myDebug or masterDebug then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_EVALUATE ***')
        end
        local WOF_INST = self.WOF_STUFF.INST        
        local WOF_PROJ = self.WOF_STUFF.PROJ
        local WOF_UNIT = self.WOF_STUFF.UNIT
        local dmg = WOF_PROJ.DamageAmount or amount 
        local health = self:GetHealth()       
        local maxHealth = self:GetMaxHealth()         
        if myDebug or masterDebug then
        	LOG('	Damage Type: ', damageType or 'Damage Type Unknown or is Nuke Weapon' )
            LOG('	WOF_PROJ.INSTIGATOR_TECH:', WOF_INST.TECH or 'Defaulting to Tech level 1 as Proj data is not avalible or is a nuke weapon' )
        	LOG('	WOF_UNIT.TECH:', WOF_UNIT.TECH or 'Defaulting to Tech level 1 as Proj data is not avalible or is a nuke weapon' )
        	LOG('	DMG:', dmg )
            LOG('	Health: ', health )
            LOG('	MaxHealth: ', maxHealth )                      	        	
        end                               
        WOF_UNIT.THRESHOLD = ( WOF_PROJ.INSTIGATOR_TECH or 1 ) / ( WOF_UNIT.TECH or 1 )
        local excess = (dmg - health) 
        if myDebug or masterDebug then      
            LOG('	Threshold: ', WOF_UNIT.THRESHOLD )        
            LOG('	Excess: ', excess )
        end                            
        WOF_UNIT.OVERKILL = excess / maxHealth
        if myDebug or masterDebug then
            LOG('	UNIT.OVERKILL: ', WOF_UNIT.OVERKILL )
        end                		   		
		if WOF_UNIT.OVERKILL > WOF_UNIT.THRESHOLD then
			if myDebug or masterDebug then		
				LOG('	Threshold exceeded, seting self.WOF_STUFF.UNIT.READY_TO_LAUNCH to true')
			end
			--Set unit flag true, thus allowing WOF event to trigger during DEATH_THREAD
			WOF_UNIT.READY_TO_LAUNCH = true
		else
			if myDebug or masterDebug then
				LOG('	Threshold not exceeded')
			end		
		end
    end
     
    function WOF_LAUNCH( self )
        local myDebug = true
        if myDebug or masterDebug then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_LAUNCH ***')
        end
                     
        -- If our unit is in transport, release it for WOF event 
        if self.WOF_STUFF.UNIT.IN_TRANSPORT then
            self:DetachFrom(true)                        
        end
        if self and self.WOF_STUFF then
        	self.WOF_STUFF.UNIT.ACTIVE = true
        	
	        --Create helper projectile 	                   
            self.WOF_STUFF.UNIT.HELPER_PROJ = self:CreateProjectileAtBone('/mods/4DC_WOF_V0.8/projectiles/WOF_helper/WOF_helper_proj.bp', self.WOF_STUFF.UNIT.BONE)  			 		
	 		--Kick off WOF event
	 		ForkThread( WOF_PROJ_UTILS.WOF_START, self.WOF_STUFF.UNIT.HELPER_PROJ, self )	
	 	elseif myDebug or masterDebug then
            LOG('	Self or WOF_STUFF is missing, aborting WOF launch') 	
 		end
	end    
           
    function WOF_CHECK_IS_ALLOWED(unit)-- Verified good 9/24/2013
    	--Checks the units blueprint cats for NOWOF and returns the result
        local myDebug = false
        if myDebug or masterDebug then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_CHECK_IS_ALLOWED ***')
        end
    	--Use a copied table for testing cats.
		local unitcats = unit:GetBlueprint().Categories or {}
		--Make sure we have cats before we check it
		if table.getn(unitcats) > 0 then
			--Check the units BP for NOWOF cat and return proper result
			if table.find(unitcats, 'NOWOF') then
		        if myDebug or masterDebug then
		            LOG('	Returning false')
		        end			
		        return false
		    else
		        if myDebug or masterDebug then
		            LOG('	Returning true')
		        end				    
		    	return true
		    end
		else
			if masterDebug then
				LOG('	WOF_UNIT_UTILS.lua, WOF_CHECK_IS_ALLOWED, Unit has no categories ' .. repr(bp.BlueprintId))
			end
		end           
    end
   
    function WOF_ID_MY_UNIT( unit )-- Verified good 10/16/2013
    	--Simple log function to aid in unit identification, outputs is in LOG format
        local myDebug = false
        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
            LOG('*** WOF_UNIT_UTILS.lua, WOF_ID_MY_UNIT ***')
        end
        if unit and unit.WOF_STUFF.UNIT and GetGameTimeSeconds() > 10 then
        	if unit.WOF_STUFF.UNIT.ID == nil or 'UNKNOWN' then
        		unit.WOF_STUFF.UNIT.ID = unit:GetEntityId()
        	end
        	if unit.WOF_STUFF.UNIT.NAME == nil or 'UNKNOWN' then
        		unit.WOF_STUFF.UNIT.NAME = unit:GetBlueprint().General.UnitName
        	end
        	if unit.WOF_STUFF.UNIT.DESRIPTION == nil or 'UNKNOWN' then
        		unit.WOF_STUFF.UNIT.DESRIPTION = unit:GetBlueprint().Description 
        	end           	        	     	
        	LOG('    Entity ID:', unit.WOF_STUFF.UNIT.ID or 'UNKNOWN' )
        	LOG('    Entity Name:', unit.WOF_STUFF.UNIT.NAME or 'UNKNOWN')
        	LOG('    Entity Description:', unit.WOF_STUFF.UNIT.DESRIPTION or 'UNKNOWN')
        else
	        if myDebug or masterDebug and GetGameTimeSeconds() > 10 then
	            LOG('	Unable to ID your unit')
	        end             
        end
    end
                    
	function WOF_DESTRUCT_TOGGLE( data ) -- Verified good 10/12/2013
		--Disable WOF for the self-destructing unit when goggled on and re-enable it if destruct is later re-toggled off.
        local myDebug = false	
	    if data.owner != -1 then
	        local unitEntities = {}
	        for _, unitId in data.units do
	            local unit = GetEntityById(unitId)
	            if OkayToMessWithArmy(unit:GetArmy()) then
	                table.insert(unitEntities, unit)
	            end
	        end
	        if table.getsize(unitEntities) > 0 then
	            local togglingOff = false
	            for _, unitEnt in unitEntities do
	            	local unit = unitEnt
	                if unit.SelfDestructThread then               				    
	                    --Provide for a means to reset the WOF flag if self destruct is cancled
					    if unit.WOF_STUFF.UNIT.ALLOWED == false then unit.WOF_STUFF.UNIT.ALLOWED = WOF_CHECK_IS_ALLOWED( unit )end
						if myDebug or masterDebug then	        
					        LOG('*** WOF_UNIT_UTILS.lua, destruct toggle has been disabled')
					        LOG('     WOF_STUFF.ALLOWED:', unit.WOF_STUFF.UNIT.ALLOWED)
					    end	                    
	                    togglingOff = true
		            elseif not togglingOff then
	                    --Prevent the triggerig of an WOF event if the unit is self destructing
	                    if unit.WOF_STUFF.UNIT.ALLOWED == true then unit.WOF_STUFF.UNIT.ALLOWED = false end
						if myDebug or masterDebug then	        
					        LOG('*** WOF_UNIT_UTILS.lua, destruct toggle has been enabled')
					        LOG('     WOF_STUFF.ALLOWED:', unit.WOF_STUFF.UNIT.ALLOWED)
					    end				    
	                end
	            end
	        end
		end
	end     
    
end--(end of non-destructive hook)