------------------------------------------------------------------------------------------
--
--  File     :  /mods/4DC_WOF_V0.8/projectiles/Sinker_helper/Sinker_helper_script.lua
--
--  Author(s):  Resin_Smoker, Domino
--
--  Summary  :  Sink Helper Projectile added for Weight of Fire (W.O.F.)
--
--  Note     :  Bone 0 = 'Sink_helper',  Bone 1 = 'anchor'  
--    
--  Copyright  2011 4DC_V0.8  All rights reserved.
--
------------------------------------------------------------------------------------------  

local EffectTemplate = import('/lua/EffectTemplates.lua')
local EffectUtil = import('/lua/EffectUtilities.lua') 
local explosion = import('/lua/defaultexplosions.lua')
local RandomFloat = import('/lua/utilities.lua').GetRandomFloat
local Sink_Helper_Proj = import('/lua/sim/projectile.lua').Projectile

Sink_helper = Class(Sink_Helper_Proj) {

    OnCreate = function(self, inWater)
        Sink_Helper_Proj.OnCreate(self)
        -- Hide the helper from view 
        self:SetVizToAllies('Never') 
        self:SetVizToFocusPlayer('Never') 
        self:SetVizToNeutrals('Never')          
        -- Kick off the WOF mechanics 
        self:ForkThread(self.Sink_Effects)                
    end,    
    
    Sink_Effects = function(self)
        -- Ensure the data has passed and that we have a target 
        if not self.Data or table.getsize(self.Data) < 3 then 
            WARN('*** Sinker Helper, Data either failed to pass or target no longer exists ***')           
            WARN('    Bone ', self.Data.TargetBone, ' Bone Valid? ', self:IsValidBone(self.Data.TargetBone))         
            WARN('    Target ', self.Data.TargetEntity)
            self:Destroy()            
        end                  
        -- Simplify things a bit
        local target = self.Data.TargetEntity            
        -- Add table for sink effects
        target.FXSinkBag = {}           
        -- Ensure the target is still viable         
        if not target:IsDead() or target:BeenDestroyed() then
            self:Destroy()          
        else                         
            -- Line up just under the target   
            Warp( self, target:CalculateWorldPositionFromRelative({0, 0, 0}) , target:GetOrientation() ) 
            local vol = explosion.GetUnitVolume(target)
            local rs = Random(vol/2, vol*2) / (vol*2)  
            -- Attach the target to the helper projectile
            if self:IsValidBone(self.Data.TargetBone) then
                target:AttachBoneTo(self.Data.TargetBone, self, 'anchor')  
            else
                WARN('Bone for unit is not valid')
                target:Destroy()
                self:Destroy()    
            end
            -- Set IsAttached_Flag
            target.IsAttached_Flag = true              
            -- Add sink FX trails  
            table.insert(target.FXSinkBag,CreateAttachedEmitter( target, 0, self:GetArmy(), '/effects/emitters/destruction_underwater_sinking_wash_01_emit.bp'):ScaleEmitter( rs ) )                                                                  
            if table.find(target:GetBlueprint().Categories,'AIR') then
                -- Stop the sound FX
                target:StopUnitAmbientSound( 'ActiveLoop' )                                               
            elseif table.find(target:GetBlueprint().Categories,'NAVAL') then
                -- add ship rotator effects here
            else
                -- for later use with other units        
            end                    
        end                                                                                                                                           
    end,       
           
    OnImpact = function(self, targetType, targetEntity)
        -- Finish off death event after the unit touches the sea floor
        if self.Data.TargetEntity then
            local target = self.Data.TargetEntity 
            if target:IsDead() and not target:BeenDestroyed() and targetType == 'Terrain' then
                if target:GetBlueprint().Audio.Destroyed then
                    target:PlayUnitSound('Destroyed')
                end
                -- Remove trails FX if they still exsist
                if target.FXTrailBag then 
                    EffectUtil.CleanupEffectBag(target,'FXSinkBag')      
                end
                -- Add a simple underwater explosion                 
                for k, v in EffectTemplate.DefaultProjectileUnderWaterImpact do                 
                    target.UnderWaterDeathFX = CreateEmitterAtEntity( target, self:GetArmy(), v ):ScaleEmitter( explosion.GetUnitVolume(target) * 2 )
                    target.Trash:Add(target.UnderWaterDeathFX)
                end 
                target:CreateWreckage( target.OverKillRatio )                                            
                target:Destroy()
            end       
        end    
        self:Destroy()
    end,       
}          
TypeClass = Sink_helper